@@ -7,7 +7,7 @@ from itertools import groupby
 
 import aioredis
 
-from aioredlock.errors import LockError
+from aioredlock.errors import LockError, LockAcquiringError, LockRuntimeError
 from aioredlock.sentinel import Sentinel
 from aioredlock.utility import clean_password
 
@@ -18,6 +18,16 @@ def all_equal(iterable):
     return next(g, True) and not next(g, False)
 
 
+def raise_error(results, default_message):
+    errors = [e for e in results if isinstance(e, BaseException)]
+    if any(type(e) is LockRuntimeError for e in errors):
+        raise [e for e in errors if type(e) is LockRuntimeError][0]
+    elif any(type(e) is LockAcquiringError for e in errors):
+        raise [e for e in errors if type(e) is LockAcquiringError][0]
+    else:
+        raise LockError(default_message) from errors[0]
+
+
 class Instance:
 
     # KEYS[1] - lock resource key
@@ -118,7 +128,7 @@ class Instance:
             self.set_lock_script_sha1,
             self.unset_lock_script_sha1,
             self.get_lock_ttl_script_sha1,
-        ) = (r.decode() for r in await asyncio.gather(*tasks))
+        ) = (r.decode() if isinstance(r, bytes) else r for r in await asyncio.gather(*tasks))
 
     async def connect(self):
         """
@@ -195,11 +205,11 @@ class Instance:
                 return await self.set_lock(resource, lock_identifier, lock_timeout, register_scripts=True)
             self.log.debug('Can not set lock "%s" on %s',
                            resource, repr(self))
-            raise LockError('Can not set lock') from exc
+            raise LockAcquiringError('Can not set lock') from exc
         except (aioredis.errors.RedisError, OSError) as exc:
             self.log.error('Can not set lock "%s" on %s: %s',
                            resource, repr(self), repr(exc))
-            raise LockError('Can not set lock') from exc
+            raise LockRuntimeError('Can not set lock') from exc
         except asyncio.CancelledError:
             self.log.debug('Lock "%s" is cancelled on %s',
                            resource, repr(self))
@@ -233,11 +243,11 @@ class Instance:
                 return await self.get_lock_ttl(resource, lock_identifier, register_scripts=True)
             self.log.debug('Can not get lock "%s" on %s',
                            resource, repr(self))
-            raise LockError('Can not get lock') from exc
+            raise LockAcquiringError('Can not get lock') from exc
         except (aioredis.errors.RedisError, OSError) as exc:
             self.log.error('Can not get lock "%s" on %s: %s',
                            resource, repr(self), repr(exc))
-            raise LockError('Can not get lock') from exc
+            raise LockRuntimeError('Can not get lock') from exc
         except asyncio.CancelledError:
             self.log.debug('Lock "%s" is cancelled on %s',
                            resource, repr(self))
@@ -271,11 +281,11 @@ class Instance:
                 return await self.unset_lock(resource, lock_identifier, register_scripts=True)
             self.log.debug('Can not unset lock "%s" on %s',
                            resource, repr(self))
-            raise LockError('Can not unset lock') from exc
+            raise LockAcquiringError('Can not unset lock') from exc
         except (aioredis.errors.RedisError, OSError) as exc:
             self.log.error('Can not unset lock "%s" on %s: %s',
                            resource, repr(self), repr(exc))
-            raise LockError('Can not set lock') from exc
+            raise LockRuntimeError('Can not unset lock') from exc
         except asyncio.CancelledError:
             self.log.debug('Lock "%s" unset is cancelled on %s',
                            resource, repr(self))
@@ -324,8 +334,8 @@ class Redis:
         :param lock_timeout: lock's lifetime
         :return float: The elapsed time that took to lock the instances
             in seconds
-        :raises: LockError if the lock has not been set to at least (N/2 + 1)
-            instances
+        :raises: LockRuntimeError or LockAcquiringError or LockError if the lock has not
+            been set to at least (N/2 + 1) instances
         """
         start_time = time.monotonic()
 
@@ -336,13 +346,13 @@ class Redis:
         successful_sets = sum(s is None for s in successes)
 
         elapsed_time = time.monotonic() - start_time
-        locked = True if successful_sets >= int(len(self.instances) / 2) + 1 else False
+        locked = successful_sets >= int(len(self.instances) / 2) + 1
 
         self.log.debug('Lock "%s" is set on %d/%d instances in %s seconds',
                        resource, successful_sets, len(self.instances), elapsed_time)
 
         if not locked:
-            raise LockError('Can not acquire the lock "%s"' % resource)
+            raise_error(successes, 'Can not acquire the lock "%s"' % resource)
 
         return elapsed_time
 
@@ -353,8 +363,8 @@ class Redis:
         :param resource: The resource string name to fetch
         :param lock_identifier: The id of the lock. A unique string
         :return float: The TTL of that lock reported by redis
-        :raises: LockError if the lock has not been set to at least (N/2 + 1)
-            instances
+        :raises: LockRuntimeError or LockAcquiringError or LockError if the lock has not
+            been set to at least (N/2 + 1) instances
         """
         start_time = time.monotonic()
         successes = await asyncio.gather(*[
@@ -363,7 +373,7 @@ class Redis:
         ], return_exceptions=True)
         successful_list = [s for s in successes if not isinstance(s, Exception)]
         # should check if all the value are approx. the same with math.isclose...
-        locked = True if len(successful_list) >= int(len(self.instances) / 2) + 1 else False
+        locked = len(successful_list) >= int(len(self.instances) / 2) + 1
         success = all_equal(successful_list) and locked
         elapsed_time = time.monotonic() - start_time
 
@@ -371,7 +381,7 @@ class Redis:
                        resource, len(successful_list), len(self.instances), elapsed_time)
 
         if not success:
-            raise LockError('Could not fetch the TTL for lock "%s"' % resource)
+            raise_error(successes, 'Could not fetch the TTL for lock "%s"' % resource)
 
         return successful_list[0]
 
@@ -382,8 +392,8 @@ class Redis:
         :param resource: The resource string name to lock
         :param lock_identifier: The id of the lock. A unique string
         :return float: The elapsed time that took to lock the instances in iseconds
-        :raises: LockError if the lock has not matching identifier in more then
-            (N/2 - 1) instances
+        :raises: LockRuntimeError or LockAcquiringError or LockError if the lock has no
+            matching identifier in more then (N/2 - 1) instances
         """
         start_time = time.monotonic()
 
@@ -394,13 +404,13 @@ class Redis:
         successful_removes = sum(s is None for s in successes)
 
         elapsed_time = time.monotonic() - start_time
-        unlocked = True if successful_removes >= int(len(self.instances) / 2) + 1 else False
+        unlocked = successful_removes >= int(len(self.instances) / 2) + 1
 
         self.log.debug('Lock "%s" is unset on %d/%d instances in %s seconds',
                        resource, successful_removes, len(self.instances), elapsed_time)
 
         if not unlocked:
-            raise LockError('Can not release the lock')
+            raise_error(successes, 'Can not release the lock')
 
         return elapsed_time
 
@@ -418,9 +428,7 @@ class Redis:
         ], return_exceptions=True)
         successful_sets = sum(s is True for s in successes)
 
-        locked = True if successful_sets >= int(len(self.instances) / 2) + 1 else False
-
-        return locked
+        return successful_sets >= int(len(self.instances) / 2) + 1
 
     async def clear_connections(self):
 
